<?php

require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginOptions.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginAdmin.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginWork.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginSource.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginUpdate.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PerelinkPluginNotification.php';

class PerelinkPlugin
{

    /**
     * @var PerelinkPluginOptions
     */
    private static $options;

    /**
     * @var PerelinkPluginAdmin
     */
    private static $admin;

    /**
     * @var PerelinkPluginWork
     */
    private static $work;

    /**
     * @var PerelinkPluginSource
     */
    private static $source;

    /**
     * @var PerelinkPluginUpdate
     */
    private static $update;

    /**
     * @var PerelinkPluginNotification
     */
    private static $notification;

    public static function init($pluginFile)
    {
        if (self::$options instanceof PerelinkPluginOptions) {
            return;
        }
        self::$options = new PerelinkPluginOptions($pluginFile);
        if (is_admin()) {
            self::$admin = new PerelinkPluginAdmin();
            self::$update = new PerelinkPluginUpdate();
        }
        self::$work = new PerelinkPluginWork();
        self::$source = new PerelinkPluginSource();
        self::$notification = new PerelinkPluginNotification();
        self::registerHooks();
    }

    private static function registerHooks()
    {
        register_activation_hook(self::$options->pluginFile, array('PerelinkPlugin', 'activate'));
        register_deactivation_hook(self::$options->pluginFile, array('PerelinkPlugin', 'deactivate'));
        register_uninstall_hook(self::$options->pluginFile, array('PerelinkPlugin', 'uninstall'));
    }

    public static function activate()
    {
        self::$options->saveOptions();
    }

    public static function deactivate()
    {

    }

    public static function uninstall()
    {
        self::$options->deleteOptions();
    }

    /**
     *
     * @return PerelinkPluginOptions
     */
    public static function getOptions()
    {
        return self::$options;
    }

    /**
     *
     * @return PerelinkPluginSource
     */
    public static function getSource()
    {
        return self::$source;
    }

    /**
     *
     * @return PerelinkPluginUpdate
     */
    public static function getUpdate()
    {
        return self::$update;
    }

    /**
     * @param array $params
     * array(
     *     'limit' => 'Количество отображаемых элементов, null (по умолчанию) - отображать все',
     *     'offset' => 'Количество элементов, которые следует пропустить, по умолчанию 0'
     * );
     * @param bool $return
     * @return string
     */
    public static function getAfterText($params = array(), $return = false)
    {
        if ($return) {
            return self::$work->getAfterText($params);
        } else {
            echo self::$work->getAfterText($params);
        }
    }

    public static function getVersion()
    {
        $data = get_plugin_data(self::$options->getPluginFile());
        return $data['Version'];
    }

    public static function getName()
    {
        $data = get_plugin_data(self::$options->getPluginFile());
        return $data['Name'];
    }

    /**
     * @return PerelinkPluginNotification
     */
    public static function getNotification()
    {
        return self::$notification;
    }

}
