<?php

class PerelinkPluginUpdate
{

    private static $infoUrl = 'https://dl.dropboxusercontent.com/u/44054592/workspace/perelink/plugin/perelink.info';
    private static $pluginUrl = 'https://dl.dropboxusercontent.com/u/44054592/workspace/perelink/plugin/perelink.zip';
    private $curl;
    private $lastVersion;

    public function check()
    {
        if(!$this->getLastVersion()){
            return true;
        }
        return version_compare(PerelinkPlugin::getVersion(), $this->getLastVersion(),'>=');
    }

    public function update()
    {
        $error = false;
        deactivate_plugins(PerelinkPlugin::getOptions()->getPluginFile());
        if (!copy(self::$pluginUrl, $this->getUploadFileName())) {
            PerelinkPlugin::getNotification()->warning('Ошибка копирования плагина');
            $error = true;
        } elseif (!$this->unzip()) {
            PerelinkPlugin::getNotification()->warning('Ошибка распаковки плагина');
            $error = true;
        } else {
            unlink($this->getUploadFileName());
            if (function_exists('wp_cache_clear_cache')) {
                wp_cache_clear_cache();
            }
            PerelinkPlugin::getNotification()->message('Обновление плагина ' . PerelinkPlugin::getName() . ' успешно завершено');
        }
        activate_plugin(PerelinkPlugin::getOptions()->getPluginFile());
        return !$error;
    }

    private function unzip()
    {
        $zip = new ZipArchive;
        if (!($zip->open($this->getUploadFileName()))) {
            return false;
        }
        if (!($zip->extractTo(dirname(__FILE__) . DIRECTORY_SEPARATOR . '..'))) {
            return false;
        }
        $zip->close();
        return true;
    }

    private function getUploadFileName()
    {
        return dirname(__FILE__) . DIRECTORY_SEPARATOR . '_perelink.zip';
    }

    public function getLastVersion()
    {
        if (empty($this->lastVersion)) {
            $this->loadLastVersion();
        }
        return $this->lastVersion;
    }

    private function loadLastVersion()
    {
        $this->lastVersion = get_transient('perelink_version');
        if($this->lastVersion === false){
            $this->curl = new CurlHelper();
            $this->curl->url = self::$infoUrl;
            $this->curl->execute();
            $this->lastVersion = $this->curl->getBody();
            set_transient('perelink_version', $this->lastVersion,60*60);
        }
    }

}
